#include <stdio.h>
#include <stdlib.h>
#include "bitmap.h"

#define OFFSET_WIDTH           18
#define OFFSET_HEIGHT          22
#define OFFSET_BITS_PER_PIXEL  28
#define OFFSET_COMPRESSION     30
#define OFFSET_PIXELS          54

bitmap_status_t bitmap_create(const char* name, bitmap_t* obj) {
    FILE* file = fopen(name, "rb");
    
    if (!file) {
        return FILE_ERROR;
    }
    
    int b0 = fgetc(file);
    int b1 = fgetc(file);
    
    if (b0 != 'B' || b1 != 'M') {
        fclose(file);
        return BAD_BMP_FILE_ERROR;
    }
    
    fread(&(obj->size), sizeof(obj->size), 1, file);
    
    obj->contents = malloc(obj->size);
    
    if (!(obj->contents)) {
        fclose(file);
        return OUT_OF_MEMORY_ERROR;
    }
    
    rewind(file);
    
    size_t bytes_read = fread(obj->contents, sizeof(uint8_t), obj->size, file);
    fclose(file);
    
    if (bytes_read != obj->size) {
        bitmap_dispose(obj);
        return BAD_BMP_FILE_ERROR;
    }
    
    uint16_t num_bits_pixel = *((uint16_t*)(obj->contents + OFFSET_BITS_PER_PIXEL));
    
    if (num_bits_pixel != 24) {
        bitmap_dispose(obj);
        return UNSUPPORTED_FILE_FORMAT_ERROR;
    }
    
    uint32_t compression = *((uint32_t*)(obj->contents + OFFSET_COMPRESSION));
    
    if (compression) {
        bitmap_dispose(obj);        
        return UNSUPPORTED_FILE_FORMAT_ERROR;
    }
    
    obj->width = *((uint32_t*)(obj->contents + OFFSET_WIDTH));
    
    if (obj->width % 4 != 0) {
        bitmap_dispose(obj);        
        return UNSUPPORTED_FILE_FORMAT_ERROR;
    }
    
    obj->height = *((uint32_t*)(obj->contents + OFFSET_HEIGHT));
    obj->num_pixels = obj->width * obj->height;
    obj->pixels = (rgb_t*)(obj->contents + OFFSET_PIXELS);
    
    return OK;
}

bitmap_status_t bitmap_dispose(bitmap_t* obj) {
    free(obj->contents);
    obj->contents = NULL;
    return OK;
}

bitmap_status_t bitmap_store(const char* name, bitmap_t* obj) {
    FILE* file = fopen(name, "wb");
    
    if (!file) {
        return FILE_ERROR;
    }
    
    size_t bytes_written = fwrite(obj->contents, sizeof(uint8_t), obj->size, file);
    
    fclose(file);
    
    if (bytes_written != obj->size) {
        return FILE_ERROR;
    }
    
    return OK;
}

